import 'package:flutter/material.dart';

abstract class GG_BaseState<T extends StatefulWidget> extends State<T> {
  ///页面标题
  String title;

  ///数据存储
  List dataArray;

  ///分页开始
  int start = 1;

  ///分页条数
  int count = 10;
  //是否加载过数据
  bool isLoading;
  //是否加载数据错误
  bool error;

  ///页面显示图片
  Image tipImage;

  ///提示信息标题
  String tipTitle;

  ///提示内容
  String tipContent;

  ///是否显示操作按钮（默认展示）
  bool isShowErrorButton = true;

  ///按钮文字（默认为重新加载）
  String errorButtonText = '重新加载';

  ///出错显示文字
  String errorShowText = '服务器出错了，正在抢修中';

  ///导航栏颜色，（默认初始值）
  Color navColor = Color.fromRGBO(19, 191, 142, 1);

  @override
  Widget build(BuildContext context) {
    if (isLoading && error) {
      //已经加载过数据，但是有错误
      return buildPage(
          Image.asset(
            "images/gg_fuwuqichucuo.png",
            package: "gg_basewidget",
          ),
          null,
          errorShowText);
    } else if (isLoading && !error && dataArray.length == 0) {
      //已经加载过数据，没有错误，页面无数据展示
      return buildPage(
          Image.asset(
            "images/gg_wushuju.png",
            package: "gg_basewidget",
          ),
          tipTitle,
          tipContent);
    } else {
      return buildBody(context);
    }
  }

  Widget buildPage(Image image, String tipTitleTemp, String tipContentTemp) {
    Size screenSize = MediaQuery.of(context).size;
    double buttonWidth = screenSize.width * 0.35;
    double buttonHeight = screenSize.width * 0.35 * 0.25;
    return Scaffold(
      appBar: AppBar(
          title: Text(
            title ?? '',
            style: TextStyle(),
          ),
          backgroundColor: navColor),
      body: Center(
        child: Column(
          children: <Widget>[
            Container(
              margin: EdgeInsets.only(top: screenSize.height * 0.18),
              width: screenSize.width,
              child: Container(
                width: screenSize.width * 0.4,
                height: screenSize.width * 0.4,
                child: image,
              ),
            ),
            Offstage(
              offstage: (tipTitle != null && tipTitle != '') ? false : true,
              child: Container(
                margin:
                    EdgeInsets.only(top: 15, bottom: 0, left: 50, right: 50),
                child: Text(
                  tipTitleTemp ?? '',
                  style: TextStyle(
                      fontSize: 18, color: Color.fromRGBO(51, 51, 51, 1)),
                  textAlign: TextAlign.center,
                ),
              ),
            ),
            Offstage(
              offstage: (tipContent != null && tipContent != '') ? false : true,
              child: Container(
                margin:
                    EdgeInsets.only(top: 15, bottom: 15, left: 50, right: 50),
                child: Text(
                  tipContentTemp ?? '',
                  style: TextStyle(
                      fontSize: 16, color: Color.fromRGBO(51, 51, 51, .8)),
                  textAlign: TextAlign.center,
                ),
              ),
            ),
            Offstage(
              offstage: !isShowErrorButton,
              child: FlatButton(
                color: Color.fromRGBO(19, 191, 142, 1),
                shape: RoundedRectangleBorder(
                  side: BorderSide.none,
                  borderRadius:
                      BorderRadius.all(Radius.circular(buttonHeight * 0.5)),
                ),
                child: Container(
                  alignment: Alignment.center,
                  width: buttonWidth,
                  height: buttonHeight,
                  child: Text(
                    errorButtonText ?? '',
                    style: TextStyle(color: Colors.white),
                  ),
                ),
                onPressed: () {
                  dggReloadData();
                },
              ),
            )
          ],
        ),
        // color: Colors.green,
      ),
    );
  }

  ///必须实现的页面
  @protected
  Widget buildBody(BuildContext context);

  ///加载失败，重新加载按钮的回调事件
  @protected
  void dggReloadData();
}
